import matplotlib.pyplot as plt
import string

# zliczamy slowa w podanym pliku
def count_words_in_file(input_file):
    word_count = {}
    with open(input_file, encoding='utf8') as f:
        for line in f:
            words = line.split()
            for word in words: # (1)
                word = word.strip(string.punctuation).casefold() # (2, 3)
                if word == '':
                    continue
                word_count[word] = word_count.get(word, 0) + 1  # (4)
    return word_count

'''
Komentarze
(1) - W tej zagniezdzonej petli iterujemy po wszystkich slowach
      w pliku. Slowami sa ciagi znakow otoczone bialymi znakami,
      a wiec moga zawierac niechciane symbole interpunkcyjne, np.
      "sea,", "What?" etc.
(2) - string.punctuation to napis zawierajacy (standardowe) symbole
      interpunkcyjne. word.strip(string.punctuation) pozbywa sie ich
      z poczatku i konca slowa
(3) - casefold() to agresywna wersja lower(), zalecana przez standard
       unicode przy porownywaniu napisow
(4) - dla kazdego (przetworzonego) slowa, aktualizujemy slownik
      stowarzyszajacy slowo z iloscia jego wystapien.
'''

# wypisujemy najpopularniejsze slowa z danego pliku (jako funkcja)     
def print_top_words(input_file, num=10):
    word_count = count_words_in_file(input_file)
    lst = [(count, word) for word, count in word_count.items()]
    lst.sort(reverse=True)
    # teraz lst to pary: ilosc wystapien i slowo,
    # uporzadkowane od najwiekszej do najmniejszej ilosci wystapien
    print(f'Top {num} slow w {input_file}:')
    for i in range(num):
        count, word = lst[i]
        print(f"{word:10} {count}")

# jak poprzednio, ale jako wykres slupkowy
def draw_top_words(input_file, num=10):
    word_count = count_words_in_file(input_file)
    lst = [(count, word) for word, count in word_count.items()]
    lst.sort(reverse=True)
    # teraz lst to pary: ilosc wystapien i slowo,
    # uporzadkowane od najwiekszej do najmniejszej ilosci wystapien
    plt.title(f'Top {num} slow w {input_file}:')
    xs = []
    ys = []
    for count, word in lst[:num]:
        xs.append(word)
        ys.append(count)
    plt.bar(xs, ys)
    plt.show()

# histogram ilosci ROZNYCH slow danej dlugosci, wystepujacych w pliku:
def draw_histogram(input_file):
    word_count = count_words_in_file(input_file)
    lst = [len(word) for word in word_count]
    # kubelki postaci [n, n+1) tak, aby kazda dlugosc miala swoj kubelek
    plt.hist(lst, bins=range(0, max(lst)+2))
    # opis
    plt.title(f'Licba roznych slow o danej dlugosci w {input_file}')
    plt.show()
    


if __name__ == "__main__":
    print_top_words('20000en.txt')
    print_top_words('southpole.txt')
    draw_top_words('prideandprejudice.txt')
    draw_histogram('politeia.txt')

